/*
   This program demonstrates how difficult it is in
   languages like Java (or C or C++) to instantiate
   a complex data structure like a binary tree. Below
   is code that instantiates the following binary tree.

                 a
               /   \
             /       \
            b         c
           / \          \
         /     \          \
        d       e          f
                          /
                        /
                       g
                        \
                          \
                            h


   This program assumes that you have installed the Graphviz
   package, which includes the dot.exe program. You can
   download the installer for Graphviz and dot.exe from this URL.

     http://www.graphviz.org/pub/graphviz/stable/windows/graphviz-2.28.0.msi
*/

public class Language_0_Example_1
{
   public static void main(String[] args)
   {
      BTree btree = null;

      // Here is how we can build a BTree data structure.
      btree = new BTree("a", new BTree("b", new BTree("d"), new BTree("e")), new BTree("c", null, new BTree("f", new BTree("g", null, new BTree("h")), null)));

      // Here is a clearer way to format the same code.
       // (Notice that this is formatted in a way analogous
       //  to the way the prettyprinter prints the btree.)
      btree = new BTree("a",
                        new BTree("b",
                                  new BTree("d"),
                                  new BTree("e")),
                        new BTree("c",
                                  null,
                                  new BTree("f",
                                            new BTree("g",
                                                      null,
                                                      new BTree("h")),
                                            null)) );


       // Here is a third way to format the same code.
      btree = new BTree("a", new BTree("b", new BTree("d"),
                                            new BTree("e")),
                             new BTree("c", null,
                                            new BTree("f", new BTree("g", null,
                                                                          new BTree("h")),
                                                           null)) );


      // Let us traverse the tree so that we can convince
      // ourselves that it was constructed correctly.
      // We should get "a b d e c f g h".
      System.out.println( " Pre-order traversal = " + Traverse.preOrder( btree ) + "\n" );
      // We should get "d b e a c g h f".
      System.out.println( "  In-order traversal = " + Traverse.inOrder( btree ) + "\n" );
      // We should get "d e b h g f c a".
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree ) + "\n" );


      // The "pretty printer" prints out the tree in a nice format.
      System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );


      // Create dot and png files from the btree.
      if(true)  // "conditional complilation" the java way
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_1";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( BTree2dot_ver1.btree2dot(btree) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }

   }//main

}//Language_0_Example_1